@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2016-2022 The pyXem developers
+# Copyright 2016-2023 The pyXem developers
 #
 # This file is part of pyXem.
 #
@@ -22,7 +22,9 @@ import numpy as np
 from pyxem.signals.tensor_field import DisplacementGradientMap
 
 
-def get_DisplacementGradientMap(strained_vectors, unstrained_vectors, weights=None):
+def get_DisplacementGradientMap(
+    strained_vectors, unstrained_vectors, weights=None, return_residuals=False, **kwargs
+):
     r"""Calculates the displacement gradient tensor at each navigation position in a map.
 
     Compares vectors to determine the 2 x 2 matrix,
@@ -43,6 +45,11 @@ def get_DisplacementGradientMap(strained_vectors, unstrained_vectors, weights=No
         basis vectors, V and U, defined as row vectors.
     weights : list
         of weights to be passed to the least squares optimiser, not used for n=2
+    return_residuals: Bool
+        If the residuals for the least squares optimiser should be returned.
+    kwargs: dict
+        Any additional keyword arguments passed to the `hyperspy.signals.BaseSignal.map`
+        function.
 
     Returns
     -------
@@ -67,12 +74,27 @@ def get_DisplacementGradientMap(strained_vectors, unstrained_vectors, weights=No
         inplace=False,
         output_signal_size=(3, 3),
         output_dtype=np.float64,
+        **kwargs
     )
 
-    return DisplacementGradientMap(D)
+    if return_residuals:
+        R = strained_vectors.map(
+            get_single_DisplacementGradientTensor,
+            Vu=unstrained_vectors,
+            weights=weights,
+            inplace=False,
+            output_dtype=np.float64,
+            return_residuals=True,
+            **kwargs
+        )
+        return DisplacementGradientMap(D), R
+    else:
+        return DisplacementGradientMap(D)
 
 
-def get_single_DisplacementGradientTensor(Vs, Vu=None, weights=None):
+def get_single_DisplacementGradientTensor(
+    Vs, Vu=None, weights=None, return_residuals=False
+):
     r"""Calculates the displacement gradient tensor from a pairs of vectors.
 
     Determines the 2 x 2 matrix, :math:`\\mathbf(L)`, that maps unstrained
@@ -90,10 +112,14 @@ def get_single_DisplacementGradientTensor(Vs, Vu=None, weights=None):
         basis vectors, V and U, defined as row vectors.
     weights : list
         of weights to be passed to the least squares optimiser
+    return_residuals: Bool
+        If the residuals for the least squares optimiser should be returned.
     Returns
     -------
     D : numpy.array
         A 3 x 3 displacement gradient tensor (measured in reciprocal space).
+    residuals : numpy.array
+        The residuals for the least squares fitting.
 
     Notes
     -----
@@ -104,6 +130,10 @@ def get_single_DisplacementGradientTensor(Vs, Vu=None, weights=None):
     get_DisplacementGradientMap()
 
     """
+    is_row_nan = np.logical_not(np.any(np.isnan(Vs), axis=1))
+    Vs = Vs[is_row_nan]
+    Vu = Vu[is_row_nan]
+
     if Vu is not None:
         if Vu.dtype == object:
             Vu = Vu[()]
@@ -111,18 +141,19 @@ def get_single_DisplacementGradientTensor(Vs, Vu=None, weights=None):
         # see https://stackoverflow.com/questions/27128688
         weights = np.asarray(weights)
         # Need vectors normalized to the unstrained region otherwise the weighting breaks down
-        Vs = (
-            np.divide(Vs, np.linalg.norm(Vu, axis=0)) * np.sqrt(weights)
-        ).T  # transpose for conventions
-        Vu = (np.divide(Vu, np.linalg.norm(Vu, axis=0)) * np.sqrt(weights)).T
+        norm = np.linalg.norm(Vu, axis=1)
+        Vs = (np.divide(Vs.T, np.linalg.norm(Vu, axis=1)) * np.sqrt(weights)).T
+        Vu = (np.divide(Vu.T, np.linalg.norm(Vu, axis=1)) * np.sqrt(weights)).T
     else:
-        Vs, Vu = Vs.T, Vu.T
+        Vs, Vu = Vs, Vu
 
-    L = np.linalg.lstsq(Vu, Vs, rcond=-1)[
-        0
-    ]  # only need the return array, see np,linalg.lstsq doc
+    L, residuals, rank, s = np.linalg.lstsq(Vu, Vs, rcond=-1)
+    # only need the return array, see np,linalg.lstsq doc
     # Put caculated matrix values into 3 x 3 matrix to be returned.
     D = np.eye(3)
     D[0:2, 0:2] = L
 
-    return D
+    if return_residuals:
+        return residuals
+    else:
+        return D
