@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2016-2022 The pyXem developers
+# Copyright 2016-2023 The pyXem developers
 #
 # This file is part of pyXem.
 #
@@ -19,7 +19,11 @@
 import pytest
 import numpy as np
 
-from pyxem.utils.correlation_utils import _correlation, _get_interpolation_matrix
+from pyxem.utils.correlation_utils import (
+    _correlation,
+    _get_interpolation_matrix,
+    _wrap_set_float,
+)
 
 
 class TestCorrelations:
@@ -53,6 +57,14 @@ class TestCorrelations:
         result[0::2, :] = 5
         np.testing.assert_array_almost_equal(c, result)
 
+    def test_wrap_set_float(self):
+        target = np.zeros(20)
+        assert 2 == np.sum(_wrap_set_float(target, bottom=10.0, top=15.5, value=2))
+        target = np.zeros(20)
+        assert 2 == np.sum(_wrap_set_float(target, bottom=10.0, top=10.5, value=2))
+        target = np.zeros(20)
+        assert 2 == np.sum(_wrap_set_float(target, bottom=10.0, top=10.0, value=2))
+
     def test_correlations_normalization(self, ones_hundred):
         c = _correlation(ones_hundred, axis=0, normalize=True)
         result = np.zeros((10, 20))
@@ -85,14 +97,29 @@ class TestCorrelations:
         result[1::2, :] = -0.93478899
         np.testing.assert_array_almost_equal(c, result)
 
-    @pytest.mark.parametrize("angles", np.array([[0.1],
-                                                 [0.25, ],
-                                                 [0.5, ],
-                                                 [.9, ]]))
-    @pytest.mark.parametrize("angular_range", [0,4*np.pi/90])
-    def test_get_interpolation_matrix(self, angles,angular_range):
+    @pytest.mark.parametrize(
+        "angles",
+        np.array(
+            [
+                [0.1],
+                [
+                    0.25,
+                ],
+                [
+                    0.5,
+                ],
+                [
+                    0.9,
+                ],
+            ]
+        ),
+    )
+    @pytest.mark.parametrize("angular_range", [0, 4 * np.pi / 90])
+    def test_get_interpolation_matrix(self, angles, angular_range):
         m = np.zeros((10, 20))
         m[2:4, :] = 1
-        c = _get_interpolation_matrix(angles, num_points=90, angular_range=angular_range)
+        c = _get_interpolation_matrix(
+            angles, num_points=90, angular_range=angular_range
+        )
         com = np.average(range(90), weights=c)
-        np.testing.assert_almost_equal(com, 90*angles[0])
+        np.testing.assert_almost_equal(com, 90 * angles[0])
