@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2016-2022 The pyXem developers
+# Copyright 2016-2023 The pyXem developers
 #
 # This file is part of pyXem.
 #
@@ -128,7 +128,7 @@ def _get_chunking(signal, chunk_shape=None, chunk_bytes=None):
     if chunk_shape is not None:
         if not isiterable(chunk_shape):
             chunk_shape = [chunk_shape] * nav_dim
-    
+
     chunks_dict = {}
     for i in range(nav_dim):
         if chunk_shape is None:
@@ -520,114 +520,6 @@ def _threshold_array(dask_array, threshold_value=1, mask_array=None):
     return thresholded_array
 
 
-def _template_match_binary_image_single_frame(frame, binary_image):
-    """Template match a binary image (template) with a single image.
-
-    Parameters
-    ----------
-    frame : NumPy 2D array
-    binary_image : NumPy 2D array
-        Must be smaller than frame
-
-    Returns
-    -------
-    template_match : NumPy 2D array
-        Same size as frame
-
-    Examples
-    --------
-    >>> frame = np.random.randint(1000, size=(256, 256))
-    >>> from skimage import morphology
-    >>> binary_image = morphology.disk(4, np.uint16)
-    >>> import pyxem.utils.dask_tools as dt
-    >>> template_match = dt._template_match_binary_image_single_frame(
-    ...     frame, binary_image)
-
-    """
-    template_match = match_template(frame, binary_image, pad_input=True)
-    template_match = template_match - np.min(template_match)
-    return template_match
-
-
-def _template_match_binary_image_chunk(data, binary_image):
-    """Template match a circular disk with a 4D dataset.
-
-    Parameters
-    ----------
-    data : NumPy 4D array
-    binary_image : NumPy 2D array
-        Must be smaller than the two last dimensions in data
-
-    Returns
-    -------
-    template_match : NumPy 4D array
-        Same size as input data
-
-    Examples
-    --------
-    >>> data = np.random.randint(1000, size=(10, 10, 256, 256))
-    >>> from skimage import morphology
-    >>> binary_image = morphology.disk(4, np.uint16)
-    >>> import pyxem.utils.dask_tools as dt
-    >>> template_match = dt._template_match_binary_image_chunk(
-    ...     data, binary_image)
-
-    """
-    output_array = np.zeros_like(data, dtype=np.float32)
-    frame = np.zeros(data.shape[-2:])
-    for index in np.ndindex(data.shape[:-2]):
-        islice = np.s_[index]
-        frame[:] = data[islice]
-        output_array[islice] = _template_match_binary_image_single_frame(
-            frame, binary_image
-        )
-    return output_array
-
-
-def _template_match_with_binary_image(dask_array, binary_image):
-    """Template match a dask array with a binary image (template).
-
-    Parameters
-    ----------
-    dask_array : Dask array
-        The two last dimensions are the signal dimensions. Must have at least
-        2 dimensions.
-    binary_image : 2D NumPy array
-        Must be smaller than the two last dimensions in dask_array
-
-    Returns
-    -------
-    template_match : Dask array
-        Same size as input data
-
-    Examples
-    --------
-    >>> data = np.random.randint(1000, size=(20, 20, 256, 256))
-    >>> import dask.array as da
-    >>> dask_array = da.from_array(data, chunks=(5, 5, 128, 128))
-    >>> from skimage import morphology
-    >>> binary_image = morphology.disk(4, np.uint16)
-    >>> import pyxem.utils.dask_tools as dt
-    >>> template_match = dt._template_match_with_binary_image(
-    ...     dask_array, binary_image)
-
-    """
-    if len(binary_image.shape) != 2:
-        raise ValueError(
-            "binary_image must have two dimensions, not {0}".format(
-                len(binary_image.shape)
-            )
-        )
-    dask_array_rechunked = _rechunk_signal2d_dim_one_chunk(dask_array)
-    output_array = da.map_blocks(
-        _template_match_binary_image_chunk,
-        dask_array_rechunked,
-        binary_image,
-        dtype=np.float32,
-    )
-    return output_array
-
-
 def _peak_find_dog_single_frame(
     image,
     min_sigma=0.98,
@@ -983,159 +875,6 @@ def _center_of_mass_array(dask_array, threshold_value=None, mask_array=None):
     return beam_shifts
 
 
-def _remove_bad_pixels(dask_array, bad_pixel_array):
-    """Replace values in bad pixels with mean of neighbors.
-
-    Parameters
-    ----------
-    dask_array : Dask array
-        Must be at least two dimensions
-    bad_pixel_array : array-like
-        Must either have the same shape as dask_array,
-        or the same shape as the two last dimensions of dask_array.
-
-    Returns
-    -------
-    data_output : Dask array
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> s = pxm.dummy_data.dummy_data.get_dead_pixel_signal(lazy=True)
-    >>> dead_pixels = dt._find_dead_pixels(s.data)
-    >>> data_output = dt._remove_bad_pixels(s.data, dead_pixels)
-
-    """
-    if len(dask_array.shape) < 2:
-        raise ValueError(
-            "dask_array {0} must be at least 2 dimensions".format(dask_array.shape)
-        )
-    if bad_pixel_array.shape == dask_array.shape:
-        pass
-    elif bad_pixel_array.shape == dask_array.shape[-2:]:
-        temp_array = da.zeros_like(dask_array)
-        bad_pixel_array = da.add(temp_array, bad_pixel_array)
-    else:
-        raise ValueError(
-            "bad_pixel_array {0} must either 2-D and have the same shape "
-            "as the two last dimensions in dask_array {1}. Or be "
-            "the same shape as dask_array {2}".format(
-                bad_pixel_array.shape, dask_array.shape[-2:], dask_array.shape
-            )
-        )
-    dif0 = da.roll(dask_array, shift=1, axis=-2)
-    dif1 = da.roll(dask_array, shift=-1, axis=-2)
-    dif2 = da.roll(dask_array, shift=1, axis=-1)
-    dif3 = da.roll(dask_array, shift=-1, axis=-1)
-
-    dif = (dif0 + dif1 + dif2 + dif3) / 4
-    dif = dif * bad_pixel_array
-
-    data_output = da.multiply(dask_array, da.logical_not(bad_pixel_array))
-    data_output = data_output + dif
-
-    return data_output
-
-
-def _find_dead_pixels(dask_array, dead_pixel_value=0, mask_array=None):
-    """Find pixels which have the same value for all images.
-
-    Useful for finding dead pixels.
-
-    Parameters
-    ----------
-    dask_array : Dask array
-        Must be at least 2 dimensions
-    dead_pixel_value : scalar
-        Default 0
-    mask_array : NumPy array, optional
-        Array with bool values. The True values will be masked
-        (i.e. ignored). Must have the same shape as the two
-        last dimensions in dask_array.
-
-    Returns
-    -------
-    dead_pixels : Dask array
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> s = pxm.dummy_data.dummy_data.get_dead_pixel_signal(lazy=True)
-    >>> dead_pixels = dt._find_dead_pixels(s.data)
-
-    With a mask
-
-    >>> mask_array = np.zeros((128, 128), dtype=bool)
-    >>> mask_array[:, :100] = True
-    >>> dead_pixels = dt._find_dead_pixels(s.data, mask_array=mask_array)
-
-    With a dead_pixel_value
-
-    >>> dead_pixels = dt._find_dead_pixels(s.data, dead_pixel_value=2)
-
-    """
-    if len(dask_array.shape) < 2:
-        raise ValueError("_find_dead_pixels must have at least 2 dimensions")
-    nav_dim_size = len(dask_array.shape) - 2
-    nav_axes = tuple(range(nav_dim_size))
-    data_sum = dask_array.sum(axis=nav_axes, dtype=np.int64)
-    dead_pixels = data_sum == dead_pixel_value
-    if mask_array is not None:
-        dead_pixels = dead_pixels * np.invert(mask_array)
-
-    return dead_pixels
-
-
-def _find_hot_pixels(dask_array, threshold_multiplier=500, mask_array=None):
-    """Find single pixels which have much larger values compared to neighbors.
-
-    Finds pixel which has very large value difference compared to its
-    neighbors. The functions looks at both at the direct neighbors
-    (x-1, y), and also the diagonal neighbors (x-1, y-1).
-
-    Experimental function, so use with care.
-
-    Parameters
-    ----------
-    dask_array : Dask array
-        Must be have 4 dimensions.
-    threshold_multiplier : scaler
-        Used to threshold the dif.
-    mask_array : NumPy array, optional
-        Array with bool values. The True values will be masked
-        (i.e. ignored). Must have the same shape as the two
-        last dimensions in dask_array.
-
-    """
-    if len(dask_array.shape) < 2:
-        raise ValueError("dask_array must have at least 2 dimensions")
-
-    dask_array = dask_array.astype("float64")
-    dif0 = da.roll(dask_array, shift=1, axis=-2)
-    dif1 = da.roll(dask_array, shift=-1, axis=-2)
-    dif2 = da.roll(dask_array, shift=1, axis=-1)
-    dif3 = da.roll(dask_array, shift=-1, axis=-1)
-
-    dif4 = da.roll(dask_array, shift=(1, 1), axis=(-2, -1))
-    dif5 = da.roll(dask_array, shift=(-1, 1), axis=(-2, -1))
-    dif6 = da.roll(dask_array, shift=(1, -1), axis=(-2, -1))
-    dif7 = da.roll(dask_array, shift=(-1, -1), axis=(-2, -1))
-
-    dif = dif0 + dif1 + dif2 + dif3 + dif4 + dif5 + dif6 + dif7
-    dif = dif - (dask_array * 8)
-
-    if mask_array is not None:
-        data = _mask_array(dask_array, mask_array=mask_array)
-    else:
-        data = dask_array
-    data_mean = data.mean() * threshold_multiplier
-    data_threshold = dif < -data_mean
-    if mask_array is not None:
-        mask_array = np.invert(mask_array).astype(np.float64)
-        data_threshold = data_threshold * mask_array
-    return data_threshold
-
-
 def _intensity_peaks_image_single_frame(frame, peaks, disk_r):
     """Intensity of the peaks is calculated by taking the mean value
     of the pixel values inside radius disk_r where the centers are the
@@ -1302,285 +1041,6 @@ def _intensity_peaks_image(dask_array, peak_array, disk_r):
     return output_array
 
 
-def _background_removal_single_frame_dog(frame, min_sigma=1, max_sigma=55):
-    """Background removal using difference of Gaussians.
-
-    Parameters
-    ----------
-    frame : NumPy 2D array
-    min_sigma : float
-    max_sigma : float
-
-    Returns
-    -------
-    background_removed : Numpy 2D array
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> s = pxm.dummy_data.dummy_data.get_cbed_signal()
-    >>> s_rem = dt._background_removal_single_frame_dog(s.data[0, 0])
-
-    """
-    blur_max = ndi.gaussian_filter(frame, max_sigma)
-    blur_min = ndi.gaussian_filter(frame, min_sigma)
-    return np.maximum(np.where(blur_min > blur_max, frame, 0) - blur_max, 0)
-
-
-def _background_removal_chunk_dog(data, **kwargs):
-    """Background removal using difference of Gaussians.
-
-    Parameters
-    ----------
-    data : NumPy array
-    At least 2 dimensions
-    min_sigma : float
-    max_sigma : float
-
-    Returns
-    -------
-    output_array : NumPy array
-        Same dimensions as data
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> s = pxm.dummy_data.dummy_data.get_cbed_signal()
-    >>> s_rem = dt._background_removal_chunk_dog(s.data[0:10, 0:10,:,:])
-    """
-    output_array = np.zeros_like(data, dtype=np.float32)
-    frame = np.zeros(data.shape[-2:])
-    for index in np.ndindex(data.shape[:-2]):
-        islice = np.s_[index]
-        frame[:] = data[islice]
-        output_array[islice] = _background_removal_single_frame_dog(frame, **kwargs)
-    return output_array
-
-
-def _background_removal_dog(dask_array, **kwargs):
-    """Background removal using difference of Gaussians.
-
-    Parameters
-    ----------
-    dask_array : Dask array
-        At least 2 dimensions
-    min_sigma : float
-    max_sigma : float
-
-    Returns
-    -------
-    output_array : Dask array
-        Same dimensions as dask_array
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> import dask.array as da
-    >>> s = pxm.dummy_data.get_cbed_signal()
-    >>> dask_array = da.from_array(s.data, chunks=(5,5,25, 25))
-    >>> s_rem = pxm.signals.Diffraction2D(
-    ...     dt._background_removal_dog(dask_array))
-
-    """
-    dask_array_rechunked = _rechunk_signal2d_dim_one_chunk(dask_array)
-    output_array = da.map_blocks(
-        _background_removal_chunk_dog, dask_array_rechunked, dtype=np.float32, **kwargs
-    )
-    return output_array
-
-
-def _background_removal_single_frame_median(frame, footprint=19):
-    """Background removal using median filter.
-
-    Parameters
-    ----------
-    frame : NumPy 2D array
-    footprint : float
-
-    Returns
-    -------
-    background_removed : Numpy 2D array
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> s = pxm.dummy_data.get_cbed_signal()
-    >>> s_rem = dt._background_removal_single_frame_median(s.data[0, 0])
-
-    """
-    bg_subtracted = frame - ndi.median_filter(frame, size=footprint)
-    return bg_subtracted
-
-
-def _background_removal_chunk_median(data, **kwargs):
-    """Background removal using median filter.
-
-    Parameters
-    ----------
-    data : NumPy array
-        Must be at least 2 dimensions
-    footprint : float
-
-    Returns
-    -------
-    output_array : NumPy array
-        Same dimensions as data
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> s = pxm.dummy_data.get_cbed_signal()
-    >>> s_rem = dt._background_removal_chunk_median(s.data[0:10, 0:10,:,:])
-
-    """
-    output_array = np.zeros_like(data, dtype=np.float32)
-    frame = np.zeros(data.shape[-2:])
-    for index in np.ndindex(data.shape[:-2]):
-        islice = np.s_[index]
-        frame[:] = data[islice]
-        output_array[islice] = _background_removal_single_frame_median(frame, **kwargs)
-    return output_array
-
-
-def _background_removal_median(dask_array, **kwargs):
-    """Background removal using median filter.
-
-    Parameters
-    ----------
-    dask_array : Dask array
-        Must be at least 2 dimensions
-    footprint : float
-
-    Returns
-    -------
-    output_array : Dask array
-        Same dimensions as dask_array
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> import dask.array as da
-    >>> s = pxm.dummy_data.get_cbed_signal()
-    >>> dask_array = da.from_array(s.data+1e3, chunks=(5,5,25, 25))
-    >>> s_rem = pxm.signals.Diffraction2D(
-    ...     dt._background_removal_median(dask_array), footprint=20)
-
-    """
-    dask_array_rechunked = _rechunk_signal2d_dim_one_chunk(dask_array)
-    output_array = da.map_blocks(
-        _background_removal_chunk_median,
-        dask_array_rechunked,
-        dtype=np.float32,
-        **kwargs
-    )
-    return output_array
-
-
-def _background_removal_single_frame_radial_median(frame, centre_x=128, centre_y=128):
-    """Background removal by subtracting median of pixel at the same
-    radius from the center.
-
-    Parameters
-    ----------
-    frame : NumPy 2D array
-    centre_x : int
-    centre_y : int
-
-    Returns
-    -------
-    background_removed : Numpy 2D array
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> s = pxm.dummy_data.get_cbed_signal()
-    >>> s_rem = dt._background_removal_single_frame_radial_median(s.data[0, 0])
-    """
-
-    y, x = np.indices((frame.shape))
-    r = np.hypot(x - centre_x, y - centre_y)
-    r = r.astype(int)
-    r_flat = r.ravel()
-    diff_image_flat = frame.ravel()
-    r_median = np.zeros(np.max(r) + 1, dtype=np.float64)
-
-    for i in range(len(r_median)):
-        if diff_image_flat[r_flat == i].size != 0:
-            r_median[i] = np.median(diff_image_flat[r_flat == i])
-    image = frame - r_median[r]
-
-    return image
-
-
-def _background_removal_chunk_radial_median(data, **kwargs):
-    """Background removal by subtracting median of pixel at the same
-    radius from the center.
-
-    Parameters
-    ----------
-    data : NumPy array
-        Must be at least 2 dimensions
-    centre_x : int
-    centre_y : int
-
-    Returns
-    -------
-    output_array : NumPy array
-        Same dimensions as data
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> s = pxm.dummy_data.get_cbed_signal()
-    >>> s_rem = _background_removal_chunk_radial_median(s.data[0:10, 0:10,:,:])
-    """
-    output_array = np.zeros_like(data, dtype=np.float32)
-    frame = np.zeros(data.shape[-2:])
-    for index in np.ndindex(data.shape[:-2]):
-        islice = np.s_[index]
-        frame[:] = data[islice]
-        output_array[islice] = _background_removal_single_frame_radial_median(
-            frame, **kwargs
-        )
-    return output_array
-
-
-def _background_removal_radial_median(dask_array, **kwargs):
-    """Background removal by subtracting median of pixel at the same
-    radius from the center.
-
-    Parameters
-    ----------
-    dask_array : Dask array
-        Must be at least 2 dimensions
-    centre_x : int
-    centre_y : int
-
-    Returns
-    -------
-    output_array : Dask array
-        Same dimensions as dask_array
-
-    Examples
-    --------
-    >>> import pyxem.utils.dask_tools as dt
-    >>> s = pxm.dummy_data.get_cbed_signal()
-    >>> dask_array = da.from_array(s.data+1e3, chunks=(5,5,25, 25))
-    >>> s_r = pxm.signals.Diffraction2D(
-    ...     dt._background_removal_radial_median(
-    ...     dask_array, centre_x=128, centre_y=128))
-
-    """
-    dask_array_rechunked = _rechunk_signal2d_dim_one_chunk(dask_array)
-    output_array = da.map_blocks(
-        _background_removal_chunk_radial_median,
-        dask_array_rechunked,
-        dtype=np.float32,
-        **kwargs
-    )
-    return output_array
-
-
 def _peak_refinement_centre_of_mass_frame(frame, peaks, square_size):
     """Refining the peak positions using the center of mass of the peaks.
 
